(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = global || self, global.AutoCheckElement = factory());
}(this, function () { 'use strict';

  // eslint-disable-next-line flowtype/no-weak-types
  function debounce(callback, wait) {
    let timeout;
    return function debounced() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      const self = this;

      function later() {
        clearTimeout(timeout);
        callback.apply(self, args);
      }

      clearTimeout(timeout);
      timeout = setTimeout(later, wait);
    };
  }

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  class XHRError extends Error {
    constructor(statusCode, responseText, contentType) {
      super();

      _defineProperty(this, "statusCode", void 0);

      _defineProperty(this, "responseText", void 0);

      _defineProperty(this, "contentType", void 0);

      this.statusCode = statusCode;
      this.responseText = responseText;
      this.contentType = contentType;
    }

  }

  const requests = new WeakMap();
  const previousValues = new WeakMap();
  const checkFunctions = new WeakMap();
  class AutoCheckElement extends HTMLElement {
    constructor() {
      super();
      checkFunctions.set(this, debounce(check.bind(null, this), 300));
    }

    connectedCallback() {
      const input = this.input;
      if (!input) return;
      const checkFunction = checkFunctions.get(this);
      if (!checkFunction) return;
      input.addEventListener('change', checkFunction);
      input.addEventListener('input', checkFunction);
      input.autocomplete = 'off';
      input.spellcheck = false;
    }

    disconnectedCallback() {
      const input = this.input;
      if (!input) return;
      const checkFunction = checkFunctions.get(this);
      if (!checkFunction) return;
      input.removeEventListener('change', checkFunction);
      input.removeEventListener('input', checkFunction);
      input.setCustomValidity('');
    }

    get input() {
      const input = this.querySelector('input');
      return input instanceof HTMLInputElement ? input : null;
    }

    get src() {
      const src = this.getAttribute('src');
      if (!src) return '';
      const link = this.ownerDocument.createElement('a');
      link.href = src;
      return link.href;
    }

    set src(value) {
      this.setAttribute('src', value);
    }

    get csrf() {
      return this.getAttribute('csrf') || '';
    }

    set csrf(value) {
      this.setAttribute('csrf', value);
    }

    get required() {
      return this.hasAttribute('required');
    }

    set required(required) {
      const input = this.input;
      if (!input) return;
      input.required = required;

      if (required) {
        this.setAttribute('required', '');
      } else {
        this.removeAttribute('required');
      }
    }

  }

  function check(autoCheckElement) {
    const src = autoCheckElement.src;

    if (!src) {
      throw new Error('missing src');
    }

    const csrf = autoCheckElement.csrf;

    if (!csrf) {
      throw new Error('missing csrf');
    }

    const input = autoCheckElement.input;
    if (!input) return;
    const body = new FormData();
    body.append('authenticity_token', csrf);
    body.append('value', input.value);
    const id = body.entries ? [...body.entries()].sort().toString() : null;
    if (id && id === previousValues.get(input)) return;
    previousValues.set(input, id);
    input.dispatchEvent(new CustomEvent('auto-check-send', {
      detail: {
        body
      },
      bubbles: true
    }));

    if (!input.value.trim()) {
      input.dispatchEvent(new CustomEvent('auto-check-complete', {
        bubbles: true
      }));
      return;
    }

    const always = () => {
      autoCheckElement.dispatchEvent(new CustomEvent('loadend'));
      input.dispatchEvent(new CustomEvent('auto-check-complete', {
        bubbles: true
      }));
    };

    if (autoCheckElement.required) {
      input.setCustomValidity('Verifying…');
    }

    autoCheckElement.dispatchEvent(new CustomEvent('loadstart'));
    performCheck(input, body, src).then(data => {
      autoCheckElement.dispatchEvent(new CustomEvent('load'));
      const message = data ? data.trim() : null;

      if (autoCheckElement.required) {
        input.setCustomValidity('');
      }

      input.dispatchEvent(new CustomEvent('auto-check-success', {
        detail: {
          message
        },
        bubbles: true
      }));
    }).catch(error => {
      if (autoCheckElement.required) {
        input.setCustomValidity(errorMessage(error) || 'Something went wrong');
      }

      autoCheckElement.dispatchEvent(new CustomEvent('error'));
      input.dispatchEvent(new CustomEvent('auto-check-error', {
        detail: {
          message: errorMessage(error)
        },
        bubbles: true
      }));
    }).then(always, always);
  }

  function errorMessage(error) {
    if (error.statusCode === 422 && error.responseText) {
      if (error.contentType.includes('text/html; fragment')) {
        return error.responseText;
      }
    }
  }

  function performCheck(input, body, url) {
    const pending = requests.get(input);
    if (pending) pending.abort();

    const clear = () => requests.delete(input);

    const xhr = new XMLHttpRequest();
    requests.set(input, xhr);
    xhr.open('POST', url, true);
    xhr.setRequestHeader('Accept', 'text/html; fragment');
    const result = send(xhr, body);
    result.then(clear, clear);
    return result;
  }

  function send(xhr, body) {
    return new Promise((resolve, reject) => {
      xhr.onload = function () {
        if (xhr.status >= 200 && xhr.status < 300) {
          resolve(xhr.responseText);
        } else {
          reject(new XHRError(xhr.status, xhr.responseText, xhr.getResponseHeader('Content-Type')));
        }
      };

      xhr.onerror = function () {
        reject(new XHRError(xhr.status, xhr.responseText, xhr.getResponseHeader('Content-Type')));
      };

      xhr.send(body);
    });
  }

  if (!window.customElements.get('auto-check')) {
    window.AutoCheckElement = AutoCheckElement;
    window.customElements.define('auto-check', AutoCheckElement);
  }

  return AutoCheckElement;

}));
